(*************************************************************************

     $RCSfile: SimpleGadget.mod $
  Description: Port of simplegtgadget.c from RKM:Libraries

   Created by: fjc (Frank Copeland)
    $Revision: 1.3 $
      $Author: fjc $
        $Date: 1994/08/08 16:54:30 $

  Copyright  1994, Frank Copeland.
  This example program is part of Oberon-A.
  See Oberon-A.doc for conditions of use and distribution.

  Log entries are at the end of the file.

*************************************************************************)

MODULE SimpleGadget;

(*
** $C= CaseChk       $I= IndexChk  $L= LongAdr   $N= NilChk
** $P- PortableCode  $R= RangeChk  $S= StackChk  $T= TypeChk
** $V= OvflChk       $Z= ZeroVars
*)

IMPORT
  SYS := SYSTEM,
  e   := Exec,
  u   := Utility,
  g   := Graphics,
  i   := Intuition,
  gt  := GadTools,
  IO  := StdIO;

CONST
  VersionTag = "\0$VER: SimpleGadget 1.0 (dd.mm.yy)\r\n";

  (* Gadget defines of out choosing, to be used as GadgetID's. *)
  mygadButton = 4;

VAR
  Topaz80 : g.TextAttr;

(*------------------------------------*)
PROCEDURE Init ();

BEGIN (* Init *)
  Topaz80.name := SYS.ADR ("topaz.font");
  Topaz80.ySize := 8;
  Topaz80.style := {};
  Topaz80.flags := {}
END Init;

(*
** Standard message handling loop with GadTools message handling functions
** used (GetIMsg() and ReplyIMsg()).
*)
PROCEDURE ProcessWindowEvents (win : i.WindowPtr);

  VAR
    imsg : i.IntuiMessagePtr;
    gad : i.GadgetPtr;
    terminated : BOOLEAN;
    signals : SET;

BEGIN (* ProcessWindowEvents *)
  terminated := FALSE;
  WHILE ~terminated DO
    signals := e.base.Wait ({win.userPort.sigBit});

    (* Use GetIMsg() and ReplyIMsg() for handling *)
    (* IntuiMessages with GadTools gadgets. *)
    LOOP
      IF terminated THEN EXIT END;
      imsg := gt.base.GetIMsg (win.userPort);
      IF imsg = NIL THEN EXIT END;
      IF imsg.class = {i.idcmpGadgetUp} THEN
        gad := imsg.iAddress;
        IF gad.gadgetID = mygadButton THEN
          IO.WriteStr ("Button was pressed\n")
        END
      ELSIF imsg.class = {i.idcmpCloseWindow} THEN
        terminated := TRUE
      ELSIF imsg.class = {i.idcmpRefreshWindow} THEN
        (* This handling is REQUIRED with GadTools *)
        gt.base.BeginRefresh (win);
        gt.base.EndRefresh (win, e.LTRUE)
      END;
      (* Use the toolkit message-replying function here... *)
      gt.base.ReplyIMsg (imsg)
    END;

  END;
END ProcessWindowEvents;

(*
** Prepare for using GadTools, set up gadgets and open window.
** Clean up and when done or on error.
*)
PROCEDURE GadtoolsWindow ();

  VAR
    mysc : i.ScreenPtr;
    mywin : i.WindowPtr;
    glist, gad : i.GadgetPtr;
    ng : gt.NewGadget;
    vi : gt.VisualInfo;

BEGIN (* GadtoolsWindow *)
  glist := NIL;
  mysc := i.base.LockPubScreen ("");
  IF mysc # NIL THEN
    vi := gt.base.GetVisualInfo (mysc, u.tagEnd);
    IF vi # NIL THEN
      (* GadTools gadgets require this step to be taken *)
      gad := gt.base.CreateContext (glist);

      (* create a button gadget centered below the window title *)
      ng.textAttr := SYS.ADR (Topaz80);
      ng.visualInfo := vi;
      ng.leftEdge := 150;
      ng.topEdge := 20 + mysc.wBorTop + (mysc.font.ySize + 1);
      ng.width := 100;
      ng.height := 12;
      ng.gadgetText := SYS.ADR ("Click Here");
      ng.gadgetID := mygadButton;
      ng.flags := {};
      gad := gt.base.CreateGadget (gt.buttonKind, gad, ng, u.tagEnd);

      IF gad # NIL THEN
        mywin := i.base.OpenWindowTagsA
          ( NIL,
            i.waTitle,     SYS.ADR ("GadTools Gadget Demo"),
            i.waGadgets,   glist,   i.waAutoAdjust,  e.LTRUE,
            i.waWidth,     400,     i.waInnerHeight, 100,
            i.waDragBar,   e.LTRUE, i.waDepthGadget, e.LTRUE,
            i.waActivate,  e.LTRUE, i.waCloseGadget, e.LTRUE,
            i.waIDCMP,     gt.buttonIdcmp +
                           {i.idcmpCloseWindow, i.idcmpRefreshWindow},
            i.waPubScreen, mysc,
            u.tagEnd );
        IF mywin # NIL THEN
          gt.base.RefreshWindow (mywin, NIL);
          ProcessWindowEvents (mywin);
          i.base.CloseWindow (mywin)
        END;
      END;
      (* FreeGadgets() must be called after the context has been
      ** created.  It does nothing if glist is NIL.
      *)
      gt.base.FreeGadgets (glist);
      gt.base.FreeVisualInfo (vi)
    END;
    i.base.UnlockPubScreen ("", mysc)
  END;
END GadtoolsWindow;

BEGIN (* SimpleGadget *)
  IF i.base.version >= 37 THEN
    gt.OpenLib (FALSE);
    IF gt.base.version >= 37 THEN
      GadtoolsWindow ()
    END;
  END;
END SimpleGadget.

(*************************************************************************

  $Log: SimpleGadget.mod $
  Revision 1.3  1994/08/08  16:54:30  fjc
  Release 1.4

  Revision 1.2  1994/07/03  15:10:34  fjc
  - Incorporated changes in 3.1 Interfaces

  Revision 1.1  1994/06/20  21:03:35  fjc
  Initial revision

*************************************************************************)

